package com.me.asteroids;

import com.me.asteroids.components.AsteroidComponent;
import com.me.asteroids.components.BulletComponent;
import com.me.asteroids.components.PlayerComponent;
import com.me.asteroids.events.BulletAsteroidCollisionEvent;
import com.me.asteroids.events.CollisionEvent;
import com.me.asteroids.events.PlayerASteroidCollisionEvent;
import com.me.common.ecs.Entity;

public class EventFactory {

    public static CollisionEvent getNewCollisionEvent(Entity a, Entity b) {
        boolean isEntityAPlayer = a.hasComponent(PlayerComponent.class);
        boolean isEntityBPlayer = b.hasComponent(PlayerComponent.class);
        boolean isEntityABullet = !isEntityAPlayer && a.hasComponent(BulletComponent.class);
        boolean isEntityBBullet = !isEntityBPlayer && b.hasComponent(BulletComponent.class);
        boolean isEntityAAsteroid = !isEntityAPlayer && !isEntityABullet && a.hasComponent(AsteroidComponent.class);
        boolean isEntityBAsteroid = !isEntityBPlayer && !isEntityBBullet && b.hasComponent(AsteroidComponent.class);

        if (isEntityAAsteroid || isEntityBAsteroid) {
            if (isEntityAPlayer && isEntityBAsteroid) {
                return new PlayerASteroidCollisionEvent(a, b);
            } else if (isEntityBPlayer && isEntityAAsteroid) {
                return new PlayerASteroidCollisionEvent(b, a);
            } else if (isEntityABullet && isEntityBAsteroid) {
                return new BulletAsteroidCollisionEvent(a, b);
            } else if (isEntityBBullet && isEntityAAsteroid) {
                return new BulletAsteroidCollisionEvent(b, a);
            }
        }

        return new CollisionEvent(a, b);
    }

}
