package com.me.common.ecs;

import java.util.HashMap;
import java.util.Map;

final class ComponentType {

    private static final Map<Class<? extends Component>, ComponentType> types = new HashMap<>();

    private static long nextBit = 1l;
    private static int nextId = 0;

    final long bit;
    final int id;

    private ComponentType() {
        this.bit = nextBit;
        this.id = nextId++;

        nextBit <<= 1;
    }

    boolean isInBits(long bits) {
        return (bits & bit) == bit;
    }

    @SafeVarargs
    static long getBitsFor(Class<? extends Component>... components) {
        long bits = 0l;
        for (Class<? extends Component> clazz : components) {
            bits |= getComponentType(clazz).bit;
        }
        return bits;
    }

    static ComponentType registerComponentType(Class<? extends Component> component) {
        ComponentType type = types.get(component);
        if (type != null) {
            throw new IllegalArgumentException(component.getName() + " has already been registered.");
        }
        type = new ComponentType();
        types.put(component, type);
        return type;
    }

    static ComponentType getComponentType(Class<? extends Component> component) {
        ComponentType type = types.get(component);
        if (type == null) {
            throw new IllegalArgumentException(component.getName() + " has not been registered.");
        }
        return type;
    }

}
